/*
 *  ESRealView.h
 *  Flash'In'App
 *
 *  Copyright 2009 Eltima Software. All rights reserved.
 *
 *	Flash'In'App is not free since version 2.0.
 *	If you plan to integrate it in your project, please, purchase a license at http://mac.eltima.com/flashinapp.html
 *	Questions? Please, contact Eltima at sales@eltima.com or visit http://mac.eltima.com for more info
 *  Full usage and integration guide can be found here: http://wiki.eltima.com/user-guides/flash-in-app-macos.html
 *
 *  For more details on RealPlayer methods usage, view its guide http://service.real.com/help/library/guides/extend/embed.htm
 */

#import <Cocoa/Cocoa.h>
#import "ESScriptingPluginView.h"

/*
 * play state constants
 */
typedef enum _RealPlayState
  {
      PLAYSTATE_STOPPED = 0
    , PLAYSTATE_CONTACTING
    , PLAYSTATE_BUFFERING
    , PLAYSTATE_PLAYING
    , PLAYSTATE_PAUSED
    , PLAYSTATE_SEEKING
  } RealPlayState;

/*
 * callbacks constants
 */
enum RealCallbacks
{
  Real_NoCallbacks = 0,
  Real_OnPosLength = (1 << 0),
  Real_OnErrorMessage = (1 << 1)
};

/*
 * error level constants
 */
typedef enum _RealErrorLevel
  {
    Real_PanicError = 0,
    Real_SevereError,
    Real_CriticalError,
    Real_GeneralError,
    Real_WarningError,
    Real_NoticeError,
    Real_InformationalError,
    Real_DebugError,
  }RealErrorLevel;

@interface ESRealView : ESScriptingPluginView
{
  unsigned int callbackFlags;
  BOOL wantsErrors;
}

#pragma mark -
#pragma mark ESRealView methods
/*
 * set/get what callbacks application will receive from RealPlayer
 */
- (unsigned int) callbackFlags;
- (void) setCallbackFlags:(unsigned int)aFlags;

#pragma mark -
#pragma mark RealPlayer Plugin methods
/*
 * Indicates whether the player is currently paused or stopped, or is currently playing.
 */
- (BOOL) canPlay;

/*
 * Plays the current clip. Equivalent to clicking the Play button.
 */
- (BOOL) doPlay;

/*
 * Indicates whether the player is currently playing a clip that can be paused.
 */
- (BOOL) canPause;

/*
 * Pauses the current clip. Equivalent to clicking the Pause button.
 */
- (BOOL) doPause;

/*
 * Indicates whether the current clip is playing or paused, or the clip is already stopped. This method is compatible with RealPlayer version 5.0 and later.
 */
- (BOOL) canStop;

/*
 * Stops the clip. Equivalent to clicking the Stop button. This method is compatible with RealPlayer version 5.0 and later.
 */
- (BOOL) doStop;

/*
 * Indicates the current elapsed buffering time.
 */
- (int) bufferingTimeElapsed;

/*
 * Indicates the estimated remaining buffering time.
 */
- (int) bufferingTimeRemaining;

/*
 * Gets the text of the last status message
 */
- (NSString *) lastMessage;

/*
 * Indicates the current state of the RealPlayer.
 */
- (RealPlayState) playState;


/*
 * Indicates the total length of the clip.
 */
- (int) length;

/*
 * Set/get the current position in the clip.
 * NOTE: Be sure to wait for the seek to finish before continuing with any programming that requires the clip to be at the point specified by this method.
 */
- (int) position;
- (BOOL) setPosition:(int)aPos;


/*
 * Set/get the current clip's title string.
 */
- (NSString *) title;
- (BOOL) setTitle:(NSString *)aTitle;


/*
 * Set/get the current clip's author string.
 */
- (NSString *) author;
- (BOOL) setAuthor:(NSString *)anAuthor;


/*
 * Set/get the current clip's copyright string.
 */
- (NSString *) copyright;
- (BOOL) setCopyright:(NSString *)aCopyright;


/*
 * Indicates the height of the presentation.
 */
- (int) clipHeight;

/*
 * Indicates the width of the presentation.
 */
- (int) clipWidth;


/*
 * Indicates whether error message will be shown on the screen or will be just sent to the delegate
 *
 * NOTE: if you set this flag to NO, but still want to be informed about errors, call setCallbackFlags method with Real_OnErrorMessage parameter
 * and implement realView:receiveError:withLevel:rmaCode:userCode:userString:moreInfo: method in your delegate
 */
- (BOOL) wantErrors;
- (BOOL) setWantErrors:(BOOL)aFlag;

/*
 * Indicates the error level for the last error.
 */
- (RealErrorLevel) lastErrorSeverity;

/*
 * Indicates the user error code from the last error.
 */
- (int) lastErrorUserCode;

/*
 * Gets the error string from the last error dialog.
 */
- (NSString *) lastErrorUserString;

/*
 * Gets the RMA error code from the last error.
 */
- (int) lastErrorRMACode;

/*
 * Provides the "more info" URL from the last error.
 */
- (NSURL *) lastErrorMoreInfoURL;

/*
 * Indicates whether the image is currently in its original size.
 */
- (BOOL) isOriginalSize;

/*
 * Sets the image window to its original size.
 */
- (BOOL) makeOriginalSize;
 
/*
 * Set/get the mute state.
 */
- (BOOL) isMuted;
- (BOOL) setMuted:(BOOL)aFlag;

/*
 * Set/get the volume level.
 */
- (int) volumeLevel;
- (void) setVolumeLevel:(int)aLevel;

/*
 * Indicates whether the current clip is in stereo.
 */
- (BOOL) isStereo;

@end

@interface NSObject (ESRealViewDelegate)
/*
 * This method is called when the position in the clip changes.
 *
 * aRealView - the real view that sent the message
 * aPosition - the current position of the clip, in milliseconds.
 * aLength - the length of the clip, in milliseconds.
 * NOTE: this method is called only if you set callbackFlags to Real_OnPosLength
 */
- (void) realView:(ESRealView *)aRealView changePosition:(int)aPosition withLength:(int)aLength;

/*
 * This method is called when an error occurs.
 * 
 * aRealView - the real view that sent the message
 * anError - a text description of the error.
 * aLevel - the error level for the last error. 
 * aRmaCode - the RMA error code from the last error.
 * aUserCode - the user error code from the last error.
 * anUserSrting - the error string from the last error dialog.
 * anInfoURL - the "more info" URL from the last error. This may be empty (for example, if there is no "more info" URL).
 * NOTE: this method is called only if you set callbackFlags to Real_OnErrorMessage
 */
- (void) realView:(ESRealView *)aRealView receiveError:(NSString *)anError withLevel:(RealErrorLevel)aLevel
                                                                            rmaCode:(int)aRmaCode
                                                                           userCode:(int)aUserCode
                                                                         userString:(NSString *)anUserSrting
                                                                           moreInfo:(NSURL *)anInfoURL;

/*
 * This method is called when the message from RealPlayer is received
 * E.g. if realView:changePosition:withLength: method was not implemented, you receive notification
 * about invoking the function with "OnPosLength" name with 2 arguments
 */
- (void) realView:(ESRealView *)aRealView didCallFunction:(NSString *)aName withArguments:(NSArray *)anArguments;

@end
